<?php return array(
    'root' => array(
        'name' => '__root__',
        'pretty_version' => 'dev-main',
        'version' => 'dev-main',
        'reference' => 'ff662814b51b466b32c93fe10e75bcb7145ef305',
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        '__root__' => array(
            'pretty_version' => 'dev-main',
            'version' => 'dev-main',
            'reference' => 'ff662814b51b466b32c93fe10e75bcb7145ef305',
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ergebnis/phpstan-rules' => array(
            'pretty_version' => '2.8.0',
            'version' => '2.8.0.0',
            'reference' => '30e790621fbad05573ef9cd355279fff5122e080',
            'type' => 'phpstan-extension',
            'install_path' => __DIR__ . '/../ergebnis/phpstan-rules',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'nette/utils' => array(
            'pretty_version' => 'v4.0.5',
            'version' => '4.0.5.0',
            'reference' => '736c567e257dbe0fcf6ce81b4d6dbe05c6899f96',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nette/utils',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'phpstan/extension-installer' => array(
            'pretty_version' => '1.4.3',
            'version' => '1.4.3.0',
            'reference' => '85e90b3942d06b2326fba0403ec24fe912372936',
            'type' => 'composer-plugin',
            'install_path' => __DIR__ . '/../phpstan/extension-installer',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'phpstan/phpstan' => array(
            'pretty_version' => '2.1.8',
            'version' => '2.1.8.0',
            'reference' => 'f9adff3b87c03b12cc7e46a30a524648e497758f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpstan/phpstan',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'phpstan/phpstan-strict-rules' => array(
            'pretty_version' => '2.0.3',
            'version' => '2.0.3.0',
            'reference' => '8b88b5f818bfa301e0c99154ab622dace071c3ba',
            'type' => 'phpstan-extension',
            'install_path' => __DIR__ . '/../phpstan/phpstan-strict-rules',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'tomasvotruba/type-coverage' => array(
            'pretty_version' => '2.0.2',
            'version' => '2.0.2.0',
            'reference' => 'd033429580f2c18bda538fa44f2939236a990e0c',
            'type' => 'phpstan-extension',
            'install_path' => __DIR__ . '/../tomasvotruba/type-coverage',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
    ),
);
