<?php declare(strict_types = 1);

namespace PHPStan\ExtensionInstaller;

/**
 * This class is generated by phpstan/extension-installer.
 * @internal
 */
final class GeneratedConfig
{

	public const EXTENSIONS = array (
  'ergebnis/phpstan-rules' => 
  array (
    'install_path' => '/usr/local/src/phpunit/tools/.phpstan/vendor/ergebnis/phpstan-rules',
    'relative_install_path' => '../../../ergebnis/phpstan-rules',
    'extra' => 
    array (
      'includes' => 
      array (
        0 => 'rules.neon',
      ),
    ),
    'version' => '2.8.0',
    'phpstanVersionConstraint' => '>=2.0.0.0-dev, <3.0.0.0-dev',
  ),
  'phpstan/phpstan-strict-rules' => 
  array (
    'install_path' => '/usr/local/src/phpunit/tools/.phpstan/vendor/phpstan/phpstan-strict-rules',
    'relative_install_path' => '../../phpstan-strict-rules',
    'extra' => 
    array (
      'includes' => 
      array (
        0 => 'rules.neon',
      ),
    ),
    'version' => '2.0.3',
    'phpstanVersionConstraint' => '>=2.0.4.0-dev, <3.0.0.0-dev',
  ),
  'tomasvotruba/type-coverage' => 
  array (
    'install_path' => '/usr/local/src/phpunit/tools/.phpstan/vendor/tomasvotruba/type-coverage',
    'relative_install_path' => '../../../tomasvotruba/type-coverage',
    'extra' => 
    array (
      'includes' => 
      array (
        0 => 'config/extension.neon',
      ),
    ),
    'version' => '2.0.2',
    'phpstanVersionConstraint' => '>=2.0.0.0-dev, <3.0.0.0-dev',
  ),
);

	public const NOT_INSTALLED = array (
);

	/** @var string|null */
	public const PHPSTAN_VERSION_CONSTRAINT = '>=2.0.4.0-dev, <3.0.0.0-dev';

	private function __construct()
	{
	}

}
